
#ifndef __Object_h
#define __Object_h

#include "bool.h"
#include "error.h"

typedef enum { INTEGER, FLOAT, STRING } vartype;


class Object {
public:

  /*
   * Returns the value as int, casting it if necessary
   * (Generate error if string?)
   */
  int toInt() {
    if(type == STRING)
      error(6, "Type mismatch: number needed");
    return (type == FLOAT) ? (int) floatValue : intValue;
  }

  /*
   * Returns the value as float, casting it if necessary
   * (Generate error if string?)
   */
  float toFloat() {
    if(type == STRING)
      error(6, "Type mismatch: number needed");
    return (type == FLOAT) ? floatValue : (float) intValue;
  }

  char *asString() {
    if(type != STRING)
      error(6, "Type mismatch: string needed");
    return stringValue;
  }


  void setInt(int v) {
    intValue = v;
    type = INTEGER;
  }

  void setFloat(int v) {
    floatValue = v;
    type = FLOAT;
  }

  void setString(char *s) {
    stringValue = s; // strdup(s);
    type = STRING;
  }

  bool isInteger() {
    return type == INTEGER;
  }

  bool isFloat() {
    return type == FLOAT;
  }

  bool isString() {
    return type == STRING;
  }

  //private:
  vartype type;
  union {
    int intValue;
    float floatValue;
    char *stringValue;
  };
};


// l-value = Typ und Adresse
// r-value = Typ und Wert

// an integer, float or string value
// content of a variable or result of an expression evaluation



typedef struct {
  vartype type;
  void *ptr;
} LValue;


// !factor liefert als l-value type = int, ptr = factor
// feld%(2,4) liefert Adresse des Elements
// fred liefert Adresse der Variable


#endif
