
#include "SymbolTable.h"
#include "error.h"

#include <string.h>  // strdup
#include <stdlib.h>  // free
#include <stdio.h>   // lvar stuff



int SymbolTable::search(const char *name)
{
  for(int i = 0; i < size; i++)
    if(strcmp(name, names[i]) == 0)
      return i;

  return -1;
}


Object *SymbolTable::get(const char *name)
{
  int i = search(name);
  return (i == -1) ? NULL : &objects[i];
}


Object *SymbolTable::lookup_or_create(const char *name)
{
  int i = search(name);
  if(i == -1) {
    if(size == CAPACITY)
      error(0, "Symbol table full");
    names[size] = strdup(name);  // strdup?

    // set correct type (value undefined...)
    int len = strlen(name);
    switch(name[len - 1]) {
    case '%':
      objects[size].type = INTEGER;
      break;
    case '$':
      objects[size].type = STRING;
      break;
    default:
      objects[size].type = FLOAT;
    }

    i = size++;
  }

  return &objects[i];
}


Object *SymbolTable::put(const char *name, Object var)
{
  if(var.isString())
    var.stringValue = strdup(var.stringValue);

  int i = search(name);
  if(i == -1) {
    if(size == CAPACITY)
      error(0, "Symbol table full");
    names[size] = strdup(name);  // strdup?
    i = size++;
  }
  objects[i] = var;

  return &objects[i];
}


void SymbolTable::clear()
{
  for(int i = 0; i < size; i++)
    free(names[i]);
  // also free vars...
  size = 0;
}


void SymbolTable::lvar()
{
  printf("Variables:\n");
  printf("----------\n");

  for(int i = 0; i < size; i++) {
    printf("%s\t", names[i]);
    if(objects[i].isInteger())
      printf("%d\n", objects[i].intValue);
    else if(objects[i].isFloat())
      printf("%f\n", objects[i].floatValue);
    else if(objects[i].isString())
      puts(objects[i].stringValue);
  }
}

