/*
 * File:  expr.c
 *
 * Expression parsing and evaluation routines.
 * Arithmetic is delegated to operator functions.
 */


// only expression() and factor() are needed for public use

#include <stdio.h>  // debugging
#include <string.h> // strdup

#include "Object.h"
#include "token.h"  // all operators and functions needed
#include "lex.h"
#include "error.h"

#include "basic.h"  // getVariable, expect


void unaryOperator(int op, Object *a);
void binaryOperator(int op, Object *a, Object *b);


// needs forward declaration:
void expression(Object *val);



// void getArrayElement(...)
// {
//   // read subscript list
//   expression(...);
//   while(ttype == ',')
//     expression(...);
// }


void factor(Object *val)
{
  int op;

  switch(ttype) {
  case '+':
  case '-':
  case TT_NOT:
    op = ttype;
    nextToken();
    factor(val);
    unaryOperator(op, val);  // apply unary operator on this value
    break;
  case TT_IDENTIFIER:
    // save sval
    char temp[32];
    strcpy(temp, sval);
    //printf("getVariable: %s -> %d\n", sval, val->intValue);
    nextToken();
//     if(ttype == '(')
//       getArrayElement(...);
//     else
      getVariable(temp, val);
    break;
  case TT_NUMBER:
    // distinguish between integer and float constants?
    val->setFloat(nval);
    nextToken();
    break;
  case TT_STRING:
    val->stringValue = strdup(sval);  // make copy?
    val->type = STRING;
    nextToken();
    break;
  case '(':
    nextToken();
    expression(val);
    expect(')');
    //nextToken();
    break;
//   ttype >= 0x100) {
//     int fn = ttype;
//     nextToken();
//     // jump to function...
//  }
  default:
    error(16, "Syntax error: factor expected");
  }
}


static void power_expression(Object *val)
{
  factor(val);
  
  while(ttype == '^') {
    Object val2;
    int op = ttype;
    nextToken();
    factor(&val2);
    binaryOperator(op, val, &val2);
  }
}


static void multiplicative_expression(Object *val)
{
  power_expression(val);
  
  while(ttype == '*' || ttype == '/' ||
	ttype == TT_DIV || ttype == TT_MOD) {
    Object val2;
    int op = ttype;
    nextToken();
    power_expression(&val2);
    binaryOperator(op, val, &val2);
  }
}


static void additive_expression(Object *val)
{
  multiplicative_expression(val);
  
  while(ttype == '+' || ttype == '-') {
    Object val2;
    int op = ttype;
    nextToken();
    multiplicative_expression(&val2);
    binaryOperator(op, val, &val2);
  }
}


static void conditional(Object *val)
{
  additive_expression(val);
  
  // only one group 5 operator per unbracketed expression allowed
  // => if, not while
  if(ttype == '=' || ttype == '<' || ttype == '>' ||
     ttype == TT_NE || ttype == TT_LE || ttype == TT_GE ||
     ttype == TT_SL || ttype == TT_LSR || ttype == TT_ASR) {
    Object val2;
    int op = ttype;
    nextToken();
    additive_expression(&val2);
    binaryOperator(op, val, &val2);
  }
}


static void and_expression(Object *val)
{
  conditional(val);
  
  while(ttype == TT_AND) {
    Object val2;
    nextToken();
    conditional(&val2);
    binaryOperator(TT_AND, val, &val2);
  }
}


void expression(Object *val)
{
  and_expression(val);
  
  while(ttype == TT_OR || ttype == TT_EOR) {
    Object val2;
    int op = ttype;
    nextToken();
    and_expression(&val2);
    binaryOperator(op, val, &val2);
  }
}
