
#include <math.h>
#include <stdlib.h>  // rand, srand

#include "Object.h"
#include "arg.h"
#include "expr.h"
#include "lex.h"  // nextToken()
#include "token.h"

void expect(char c);  // in basic.cpp


void exec_abs(Object *res)
{
  factor(res);
  // provide wrapper for this stuff!
  if(res->type == STRING)
    error(6, "Type mismatch: number needed");
  if(res->type == INTEGER)
    res->intValue = abs(res->intValue);
  else  // float
    res->floatValue = fabs(res->floatValue);
}


void exec_eval(Object *res)
{
  // not very clean...
  char *expr = string_expression();
  char *p = pos;    // store lex position...
  int t = ttype;    // ... and current token
  lex_start(expr);
  expression(res);  // evaluate it
  lex_start(p);     // restore
  ttype = t;
}



void exec_int(Object *res)
{
  factor(res);
  // provide wrapper for this stuff!
  if(res->type == STRING)
    error(6, "Type mismatch: number needed");
  if(res->type == FLOAT)
    // just cast it
    res->setInt((int) res->floatValue);
  // nothing to do for ints
}


/*
 *  RND function -- random numbers
 */
void exec_rnd(Object *res)
{
  static float rnd1 = 0;

  nextToken();  // skip RND -- here or before?
  if(ttype == '(') {
    int n = int_expression();
    expect(')');
    if(n < 0) {
      srand(n);
      res->setInt(n);
    }
    else if(n == 0) {
      res->setFloat(rnd1);
    }
    else if(n == 1) {
      // random number must be < 1 -> add one to RAND_MAX
      rnd1 = rand() / ((float) RAND_MAX + 1);
      res->setFloat(rnd1);
    }
    else {  // n > 1
      int m = (rand() % n) + 1;
      res->setInt(m);
    }
  }
  else {
    int r = rand();  // 0..RAND_MAX -- incompatible!
    res->setInt(r);
  }
}


void exec_sgn(Object *res)
{
  // no sign function in C...?
  float f = float_factor();
  res->setInt(f == 0 ? 0 : f < 0 ? -1 : 1);
}


void exec_sqr(Object *res)
{
  // no sign function in C...?
  float f = float_factor();
  // check range!
  res->setFloat(sqrt(f));
}


// should go in log.c:

void exec_false(Object *res)
{
  res->setInt( 0 );
}


void exec_true(Object *res)
{
  res->setInt( -1 );
}


// tri.c:

void exec_pi(Object *res)
{
  res->setFloat( 3.141592653 );
}


// var.c:

#include <time.h>
#include <string.h>  // strdup


void exec_time(Object *res)
{
  // do we have to cast anything to float before?
  int t = clock() * 100 / CLOCKS_PER_SEC;
  res->setInt(t);
}


void exec_timeString(Object *res)
{
  char buf[25];
  time_t timer;
  time(&timer);
  struct tm *timeptr = localtime(&timer);
  strftime(buf, 25, "%a,%d %b %Y.%H:%M:%S", timeptr);
  res->setString(strdup(buf));
}
